<table class="table table-hover">
    <thead>
        <tr>
            <th>ID</th>
            <th>Object Name</th>
            <th>Timestamp</th>
            <th>Model Name</th>
            <th>Temperature</th>
            <th>Prompts Used</th>
        </tr>
    </thead>
    <tbody>
        {% if logs %}
            {% for log in logs %}
            <tr class="log-row" data-prompt-hashes="{{ log.prompt_hashes }}" hx-get="/logs/{{ log.id }}" hx-target="#log-details">
                <td>{{ log.id }}</td>
                <td>{{ log.object_name }}</td>
                <td>{{ log.timestamp }}</td>
                <td>{{ log.model_name }}</td>
                <td>{{ log.temperature }}</td>
                <td>{{ log.prompt_names | safe }}</td>
            </tr>
            {% endfor %}
        {% else %}
            <tr>
                <td colspan="6">No logs found</td>
            </tr>
        {% endif %}
    </tbody>
</table>
